/*******************************************************************************
 * FlowMigrator.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.apache.commons.io.FileUtils;
import com.atsistemas.webbeans.model.v1.BeanSection.BEAN;
import com.atsistemas.webbeans.model.v1.ExecuteViewBody;
import com.atsistemas.webbeans.model.v1.State;
import com.seatojsf.parsers.JavaState;
import com.seatojsf.parsers.ViewState;
import com.seatojsf.parsers.XHTMLState;
import com.sun.java.xml.ns.javaee.FacesConfigManagedBeanScopeOrNoneType;
import com.sun.java.xml.ns.javaee.FacesConfigManagedBeanType;
import com.sun.java.xml.ns.javaee.FacesConfigType;
import com.sun.java.xml.ns.javaee.FullyQualifiedClassType;
import com.sun.java.xml.ns.javaee.JavaIdentifierType;

/**
 * Clase principal para la migración de flujo.
 * <p>
 * Genera todos los elementos asociados al webbean, la configuración de faces-config,
 * y las clases y páginas asociadas.
 */
public class FlowMigrator {

	List<WebBean> webbeans;
	SEAProjectInfo seaProjectInfo;
	JSFProjectInfo jsfProjectInfo;
	FacesConfigType facesConfig;
	FlowStateList flowStateList;
	File templateDir;
	



	public FlowMigrator(List<WebBean> webbeans, SEAProjectInfo seaProjectInfo, 
			JSFProjectInfo jsfProjectInfo, FacesConfigType facesConfig, File templateDir) {
		this.webbeans = webbeans;
		this.seaProjectInfo = seaProjectInfo;
		this.jsfProjectInfo = jsfProjectInfo;
		this.facesConfig = facesConfig;
		this.flowStateList = new FlowStateList(webbeans);
		this.templateDir = templateDir;
	}


	/**
	 * Genera el fichero faces-config.xml y todos los elementos de flujo asociados,
	 * incluyendo clases y páginas.
	 */
	public void migrate() throws IOException {
		Iterator<WebBean> it = webbeans.iterator();
		while (it.hasNext()){
			WebBean webbean = it.next();
			writeManagedBeans(webbean);
			writeFlow(webbean);			
		}
	}
	


	/**
	 * Genera y todos los elementos de flujo, parsea clases java para su uso con JSF,
	 * adapta los beans a managedBeans.
	 */
	void writeFlow(WebBean webBean) {
		FlowStateNode root = FlowStateNode.parse(webBean.getFlow(),webBean.getId(),webBean.getViews());
		ArrayList<FlowStateNode> nodes = new ArrayList<FlowStateNode>();
		nodes.add(root);
		Set<String> processed = new HashSet<String>();
		processNodes(nodes, webBean, processed);
	}

	void processNodes(Collection<FlowStateNode> nodes, WebBean webBean, Set<String> processed) {

		Iterator<FlowStateNode> itNodes = nodes.iterator();
		FlowStateNode node = null;
		Collection<FlowStateNode> children = null;
		while (itNodes.hasNext()) {
			node = itNodes.next();
			if (!processed.contains(node.getState().getId())){
				processNode(node, webBean, processed);
				children = node.getChildren();
				if ( (null!=children) && (children.size()>0)  ){
					processNodes(node.getChildren(), webBean, processed);	
				}
			}

		}
	}

	
	
	
	
	private void xhtmlGenerate(FlowStateNode node, String webContentPath, String className, FlowStateList flowStateList){
		XHTMLState xhtmlGenerator = new XHTMLState(node, webContentPath , className, flowStateList);
		xhtmlGenerator.generateFile();		
	}
	
	
	
	private void JSFJavaFileGenerate(FlowStateNode node){
		JavaState jstate = new JavaState(node,seaProjectInfo,jsfProjectInfo);
		jstate.generateJSFJavaFile();		
	}	
	

	/**
	 * Trata cada node de flujo SEA por separado para añadirlo al proyecto JSF
	 * según el tratamiento de conversión específico por nodo.
	 * @param node
	 */
	void processNode(FlowStateNode node, WebBean webBean, Set<String> processed) {
		processed.add(node.getState().getId());
		if ("view".equals(FlowStateNode.getType(node))){
			ViewState viewState = new ViewState(node, seaProjectInfo, jsfProjectInfo, templateDir);
			viewState.writeView();
		}
		else{
			String webContentPath = jsfProjectInfo.getWebcontentPath().getAbsolutePath();
			xhtmlGenerate(node, webContentPath , FlowStateNode.getClassName(node), flowStateList);
			JSFJavaFileGenerate(node);
			addManagedBean(FlowStateNode.getClassName(node), FlowStateNode.getPackageAndClass(node), "none");			
		}
	}

	/**
	 * Genera los managedBeans para su uso con JSF y los añade a faces-config
	 */
	void writeManagedBeans(WebBean webBean){
		List<BEAN> beans = webBean.getBeans();
		if (null!=beans){
			Iterator<BEAN> itBean = beans.iterator();
			BEAN bean = null;
			String beanClass = "";
			while (itBean.hasNext()) {
				bean = itBean.next();
				addManagedBean(bean.getId(), bean.getType(), "session");
				beanClass = bean.getType();
				copy(seaProjectInfo.concatSrcPathToClass(beanClass), jsfProjectInfo.concatSrcPathToClass(beanClass));
			}			
		}
		addManagedBean("ManagementInvocation", "java.util.Stack", "session");
	}
	
	
	private boolean existManagedBean(String name){
		boolean found = false;
		List<FacesConfigManagedBeanType> list = facesConfig.getManagedBeanList();
		if (null!=list){
			Iterator<FacesConfigManagedBeanType> it = list.iterator();
			while ( (!found) && (it.hasNext()) ){
				if (it.next().getManagedBeanName().getStringValue().equals(name)){
					found = true;
				}
			}			
		}
		return found;
	}
	
	@SuppressWarnings("deprecation")
	private void addManagedBean(String name, String clazz, String scope){
		if (!existManagedBean(name)){
			FacesConfigManagedBeanType mb = facesConfig.addNewManagedBean();
			JavaIdentifierType mbname = mb.addNewManagedBeanName();
			mbname.setStringValue(name);
			mb.setManagedBeanName(mbname);
			FullyQualifiedClassType mbClazz = FullyQualifiedClassType.Factory
					.newInstance();
			mbClazz.setStringValue(clazz);
			mb.setManagedBeanClass(mbClazz);
			FacesConfigManagedBeanScopeOrNoneType fScope = FacesConfigManagedBeanScopeOrNoneType.Factory
					.newInstance();
			fScope.set(scope);
			mb.setManagedBeanScope(fScope);

		}
	}
	
	
	
	private void copy(String source, String desc){
		File fileSeaBean = new File(source);
		File fileJsfBean = new File(desc);
		try {
			if (fileSeaBean.exists()){
				FileUtils.copyFile(fileSeaBean, fileJsfBean);				
			}					
		}catch (Exception e) {
			throw new RuntimeException(e);
		}
	}



}
