/*******************************************************************************
 * FlowStateNode.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.Set;
import java.util.StringTokenizer;

import com.atsistemas.webbeans.model.v1.Case;
import com.atsistemas.webbeans.model.v1.CaseMessage;
import com.atsistemas.webbeans.model.v1.CommandBody;
import com.atsistemas.webbeans.model.v1.ExecuteViewBody;
import com.atsistemas.webbeans.model.v1.Flow;
import com.atsistemas.webbeans.model.v1.IfBody;
import com.atsistemas.webbeans.model.v1.InvokeBody;
import com.atsistemas.webbeans.model.v1.State;
import com.atsistemas.webbeans.model.v1.SwitchBody;


/**
 * 
 * Elemento que contine la información de un node de flujo SEA, el id, el nodo padre,
 * los nodos hijos y el propio "state".
 * 
 */
public class FlowStateNode {

	private String id;
	private FlowStateNode parent;
	private State state;
	private Collection<FlowStateNode> childs;
	private String webbeanId; 
	private FlowStateNode root;
	private String viewPath;
	
	
	/**
	 * Ctr.
	 */
	public FlowStateNode() {
		this.id = "";
		this.parent = null;
		this.state = null;
		this.childs = new ArrayList<FlowStateNode>();
		this.webbeanId = "";
		this.root = null;
		this.viewPath = null;
	}

	/**
	 * Añadir nodo hijo.
	 * @param child nodo a añadir como hijo
	 */
	public void addChild(FlowStateNode child) {
		childs.add(child);
	}

	/**
	 * Recupera el nodo padre del nodo actual
	 * @return nodo padre
	 */
	public FlowStateNode getParent() {
		return parent;
	}

	/**
	 * Recupera el State del nodo actual, State es como llama SEA a cada elemento de nodo,
	 * este puede ser del tipo If, Switch, Command, View, Fork, Return
	 * 
	 * @return State del nodo, entiendase que es el propio nodo.
	 */
	public State getState() {
		return state;
	}
	
	
	
	/**
	 * Recupera el nodo raíz del árbol, lo equivalente al FirstState
	 * 
	 * @return nodo raíz del árbol
	 */
	public FlowStateNode getRoot() {
		return root;
	}

	/**
	 * Recupera el id del webbean al que pertenece el nodo
	 * @return
	 */
	public String getWebbeanId() {
		return webbeanId;
	}
	
	
	


	public String getViewPath() {
		return viewPath;
	}

	/**
	 * Recupera todos los hijos del nodo actual.
	 * 
	 * @return hijos del nodo actual.
	 */
	public Collection<FlowStateNode> getChildren() {
		return childs;
	}

	private boolean existChild(String id) {
		boolean exist = false;
		Iterator<FlowStateNode> it = childs.iterator();
		State st = null;
		while ((it.hasNext()) && (!exist)) {
			st = it.next().state;
			if (id.equals(st.getId())) {
				exist = true;
			}
		}
		return exist;
	}

	private State getState(String id, List<State> states) {
		State st = null;
		Iterator<State> it = states.iterator();
		boolean found = false;
		while ((it.hasNext()) && (!found)) {
			st = it.next();
			if (st.getId().equals(id)) {
				found = true;
			}
		}
		if (!found) {
			st = null;
		}
		return st;
	}
	
	
	public static FlowStateNode getChildInAllFlow(String id, FlowStateNode node, Set<Object> processed, boolean found, FlowStateNode child){
		FlowStateNode pointer = null;
		//FlowStateNode child = null;
		Collection<FlowStateNode> nodes = node.getChildren();
		Iterator<FlowStateNode> it = nodes.iterator();
		//boolean found=false;
		while ((!found) && (it.hasNext())){
			pointer = it.next();
			if (!processed.contains(pointer)){
				processed.add(pointer);				
				if (pointer.getState().getId().equals(id)){
					found = true;
					child = pointer;
				}else{
					child = getChildInAllFlow(id, pointer, processed, found, child);
				}
			}
		}
		return child;
	}
	
	
	/**
	 * A partir de la lista de hijos y el id devuelve el hijo cuyo id es el parámetro id.
	 * 
	 * @param id identificador del hijo a buscar dentro de chilNodes.
	 * @param childNodes todos los hijos de donde buscar el id.
	 * @return FlowStateNode cuyo identificador es el parámetro id.
	 */
	public static FlowStateNode getChild(String id, Collection<FlowStateNode> childNodes){
		FlowStateNode node = null;
		Iterator<FlowStateNode> it = childNodes.iterator();
		boolean found = false;
		while ((it.hasNext()) && (!found)) {
			node = it.next();
			if (node.getState().getId().equals(id)) {
				found = true;
			}
		}
		if (!found) {
			node = null;
		}
		return node;		
	}

	// -------------------------------------------------------------------------------------------------------------------------------------
	// -------------------------------------------------------------------------------------------------------------------------------------
	// -- static methods --
	// -------------------------------------------------------------------------------------------------------------------------------------
	// -------------------------------------------------------------------------------------------------------------------------------------
	
	/**
	 * Partiendo de un elmento Flow de SEA devuelve el árbol en un objeto FlowStateNode.
	 */
	public static FlowStateNode parse(Flow flow,String webbeanId, Collection<View> views) {
		FlowStateNode root = new FlowStateNode();
		String first = flow.getFirstState();
		List<State> stateList = flow.getSTATEList();
		root.id = first;
		root.parent = null;
		root.state = root.getState(first, stateList);
		root.webbeanId = webbeanId;
		root.root = root;
		if (root.getState().isSetEXECUTEVIEW()){
			root.viewPath = getPathView(root.getState().getId(), views);
		}
		Queue<FlowStateNode> fsnQueue = new LinkedList<FlowStateNode>();
		fsnQueue.add(root);
		Set<String> processed = new HashSet<String>();
		addChilds(stateList, fsnQueue, processed, views);
		return root;
	}
	
	private static String getPathView(String viewName, Collection<View> views){
		String goView = "goView:"+viewName;
		String string = "";
		View view = null;
		Iterator<View> it = views.iterator();
		boolean found = false;
		while (it.hasNext()){
			view = it.next();
			if (goView.equals(view.getName())){
				found = true;
				string  = view.getPath();
			}
		}
		return string;
	}

	private static void addChilds(List<State> stateList,Queue<FlowStateNode> fsnQueue, Set<String> processed, Collection<View> views) {
		FlowStateNode node = null;
		while (!fsnQueue.isEmpty()) {
			node = fsnQueue.remove();
			State st = node.state;
			FlowStateNode fst = null;
			if (st.isSetCOMMAND()) {
				CommandBody command = st.getCOMMAND();
				State child = node.getState(command.getGoState(), stateList);
				// -- si no existe el child se añade
				if (!node.existChild(node.state.getId())) {
					FlowStateNode p = getChildInAllFlow(child.getId(), node.root, new HashSet<Object>(),false,null); 
					if (p!=null){
						fst = p;
					}
					else{
						fst = new FlowStateNode();
						fst.id = child.getId();
						fst.parent = node;
						fst.state = child;
						fst.webbeanId = node.webbeanId;
						fst.root = node.root;						
					}
					node.addChild(fst);
					//-- solo se encola si no esta procesado:
					if (!processed.contains(fst.id)){
						fsnQueue.add(fst);
						processed.add(fst.id);
					}
				}
			}
			else if (st.isSetEXECUTEVIEW()) {
				node.viewPath = getPathView(st.getId(), views);
				ExecuteViewBody view = st.getEXECUTEVIEW();
				List<CaseMessage> cases = view.getONMESSAGEList();
				Iterator<CaseMessage> itCases = cases.iterator();
				System.out.println("addChilds  webbean::: "+node.getWebbeanId()+" stnode:::  " + st.getId());
				while (itCases.hasNext()) {
					CaseMessage m = itCases.next();
					
					if (!node.existChild(m.getGoState())) {
						//Hijos que se añaden
						// No siempre hay que hacer un new, si existe en el flujo
						FlowStateNode p = getChildInAllFlow(m.getGoState(), node.root, new HashSet<Object>(),false,null); 
						if (p!=null){
							fst = p;
						}
						else{
							fst = new FlowStateNode();
							fst.parent = node;
							fst.state = node.getState(m.getGoState(), stateList);
							fst.id = m.getGoState();
							fst.webbeanId = node.webbeanId;
							fst.root = node.root;
							//fst.viewPath = getPathView(m.getGoState(), views);
						}
						node.addChild(fst);
						System.out.println("child  webbean::: "+fst.getWebbeanId()+" stnode:::  " + fst.state.getId());		
						if (!processed.contains(fst.id)){
							fsnQueue.add(fst);
							processed.add(fst.id);
						}						
					}
					

				}

			} 
			else if (st.isSetFORK()) {
				// -- en este caso el hijo es otro flujo
				// de momento no hacer nada.
			} 
			else if (st.isSetIF()) {
				IfBody iz = st.getIF();
				if (!node.existChild(iz.getThenGoState())) {
					FlowStateNode p = getChildInAllFlow(iz.getThenGoState(), node.root, new HashSet<Object>(),false,null); 
					if (p!=null){
						fst = p;
					}
					else{
						fst = new FlowStateNode();
						fst.parent = node;
						fst.state = node.getState(iz.getThenGoState(), stateList);
						fst.id = iz.getThenGoState();
						fst.webbeanId = node.webbeanId;
						fst.root = node.root;					
					}
					node.addChild(fst);
					if (!processed.contains(fst.id)){
						fsnQueue.add(fst);
						processed.add(fst.id);
					}			
				}
				if (!node.existChild(iz.getElseGoState())) {
					
					FlowStateNode p = getChildInAllFlow(iz.getElseGoState(), node.root, new HashSet<Object>(),false,null); 
					if (p!=null){
						fst = p;
					}
					else{
						fst = new FlowStateNode();
						fst.parent = node;
						fst.state = node.getState(iz.getElseGoState(), stateList);
						fst.id = iz.getElseGoState();
						fst.webbeanId = node.webbeanId;
						fst.root = node.root;					
					}
					node.addChild(fst);
					if (!processed.contains(fst.id)){
						fsnQueue.add(fst);
						processed.add(fst.id);
					}					
				}
			} 
			else if (st.isSetINVOKE()) {
				//-- El hijo es otro flujo, pero 
				//   el gotostate no!
				InvokeBody invoke = st.getINVOKE();
				String gotostate = invoke.getGoState();
				if (!node.existChild(gotostate)) {
					FlowStateNode p = getChildInAllFlow(gotostate, node.root, new HashSet<Object>(),false,null);
					if (p!=null){
						fst = p;
					}
					else{
						fst = new FlowStateNode();
						fst.parent = node;
						fst.state = node.getState(gotostate, stateList);
						fst.webbeanId = node.webbeanId;
						fst.id = gotostate;
						fst.root = node.root;						
					}
					node.addChild(fst);
					if (!processed.contains(fst.id)){
						fsnQueue.add(fst);
						processed.add(fst.id);
					}					
				}

			} 
			else if (st.isSetRETURN()) {
				// -- el return no tiene hijos.
				

			} 
			else if (st.isSetSWITCH()) {
				SwitchBody zwitch = st.getSWITCH();
				List<Case> cases = zwitch.getCASEList();
				Iterator<Case> itCases = cases.iterator();
				while (itCases.hasNext()) {
					Case m = itCases.next();
					if (!node.existChild(m.getGoState())) {
						FlowStateNode p = getChildInAllFlow(m.getGoState(), node.root, new HashSet<Object>(),false,null);
						if (p!=null){
							fst = p;
						}
						else{
							fst = new FlowStateNode();
							fst.parent = node;
							fst.state = node.getState(m.getGoState(), stateList);
							fst.webbeanId = node.webbeanId;
							fst.id = m.getGoState();
							fst.root = node.root;						
						}
						node.addChild(fst);
						if (!processed.contains(fst.id)){
							fsnQueue.add(fst);
							processed.add(fst.id);
						}					
					}
				}
				// -- default case
				if (!node.existChild(zwitch.getDEFAULT().getGoState())) {
					FlowStateNode p = getChildInAllFlow(zwitch.getDEFAULT().getGoState(), node.root, new HashSet<Object>(),false,null);
					if (p!=null){
						fst = p;
					}
					else{
						fst = new FlowStateNode();
						fst.parent = node;
						fst.state = node.getState(zwitch.getDEFAULT().getGoState(),	stateList);
						fst.webbeanId = node.webbeanId;
						fst.id = zwitch.getDEFAULT().getGoState();
						fst.root = node.root;					
						}
					node.addChild(fst);
					if (!processed.contains(fst.id)){
						fsnQueue.add(fst);
						processed.add(fst.id);
					}					
				}
			}
		}
	}
	
	
	
	public static String getClassName(FlowStateNode node){
		String packageAndClass = FlowStateNode.getPackageAndClass(node);
		StringTokenizer st = new StringTokenizer(packageAndClass,"\\.");
		String className = "";
		while (st.hasMoreTokens()){
			className = st.nextToken();
		}
		return className;
	}
	
	
	public static String getPackageAndClass(FlowStateNode node){
		String element = "";
		State st = node.getState();
		if (st.isSetCOMMAND()){
			element = st.getCOMMAND().getCode();
		}
		else if (st.isSetEXECUTEVIEW()){
			String path = node.getViewPath();
			StringTokenizer sTokeniker = new StringTokenizer(path,"/");
			while (sTokeniker.hasMoreElements()){
				element = sTokeniker.nextToken();
			}
			element = element.replaceFirst("\\.jsp","");
		}
		else if (st.isSetFORK()){
			//-- no tengo en cuenta los fork
		}
		else if (st.isSetIF()){
			element = st.getIF().getCondition();
		}
		else if (st.isSetINVOKE()){
			element = st.getINVOKE().getTarget();
			element = "es.allianz.invoke." + node.getWebbeanId() + "To" + element;
		}
		else if (st.isSetRETURN()){
			element = st.getId();
			element = "es.allianz.ret." + node.getWebbeanId()+"_"+(element.replaceAll(" ", "").replaceAll("\t", "")+"Stack");
		}
		else if (st.isSetSWITCH()){
			element = st.getSWITCH().getCondition();
		}
		return element.replaceAll(" ", "").replaceAll("\t", "");
	}	
	
	
	
	
	
	public static String getType(FlowStateNode node){
		String type = "";
		State st = node.getState();
		if (st.isSetCOMMAND()) {
			type = "command";
		}
		else if (st.isSetEXECUTEVIEW()) {
			type = "view";
		} 
		else if (st.isSetFORK()) {
			type = "fork";
		} 
		else if (st.isSetIF()) {
			type = "if";
		} 
		else if (st.isSetINVOKE()) {
			type = "invoke";
		} 
		else if (st.isSetRETURN()) {
			type = "return";
		} 
		else if (st.isSetSWITCH()) {
			type = "switch";
		}
		return type;
	}

	public String toString() {
		return id;
	}

}
