/*******************************************************************************
 * WebXmlSEA.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.apache.xmlbeans.XmlException;
import com.sun.java.xml.ns.j2Ee.EnvEntryType;
import com.sun.java.xml.ns.j2Ee.FilterMappingType;
import com.sun.java.xml.ns.j2Ee.FilterType;
import com.sun.java.xml.ns.j2Ee.ListenerType;
import com.sun.java.xml.ns.j2Ee.ParamValueType;
import com.sun.java.xml.ns.j2Ee.ServletMappingType;
import com.sun.java.xml.ns.j2Ee.ServletType;
import com.sun.java.xml.ns.j2Ee.WebAppDocument;
import com.sun.java.xml.ns.j2Ee.XsdIntegerType;


public class WebXmlSEA {
	
	
	private Collection<EnvEntry> envEntries;
	private Collection<Filter> filters;
	private Collection<FilterMapping> filtersMapping;
	private Collection<Servlet> servlets;
	private Collection<ServletMapping> servletsMapping;
	private Collection<Listener> listeners;
	
	private WebXmlSEA(){}


	public static WebXmlSEA parse(File webXml) throws XmlException, IOException{
		WebXmlSEA dd = new WebXmlSEA();
		dd.envEntries = new ArrayList<EnvEntry>();
		dd.filters = new ArrayList<Filter>();
		dd.filtersMapping = new ArrayList<FilterMapping>();
		dd.servlets = new ArrayList<Servlet>();
		dd.servletsMapping = new ArrayList<ServletMapping>();
		dd.listeners = new ArrayList<Listener>();
		WebAppDocument document = WebAppDocument.Factory.parse(webXml);
		loadEnvEntries(dd, document);
		loadFilters(dd, document);
		loadFiltersMapping(dd, document);
		loadServlets(dd, document);
		loadServletsMapping(dd, document);
		loadListeners(dd,document);
		return dd;
	}


	private static void loadListeners(WebXmlSEA dd, WebAppDocument document) {
		List<ListenerType> listeners = document.getWebApp().getListenerList();
		ListenerType listenerType = null;
		Listener listener = null;
		Iterator<ListenerType> it = listeners.iterator();
		while (it.hasNext()){
			listenerType = it.next();
			listener = new Listener(listenerType.getListenerClass().getStringValue());
			dd.listeners.add(listener);
		}
	}


	private static void loadServletsMapping(WebXmlSEA dd,
			WebAppDocument document) {
		List<ServletMappingType> servletsMapping = document.getWebApp().getServletMappingList();
		ServletMappingType servletMappingType = null;
		ServletMapping servletMapping = null;
		Iterator<ServletMappingType> it = servletsMapping.iterator();
		while (it.hasNext()){
			servletMappingType = it.next();
			servletMapping = new ServletMapping(
					servletMappingType.getServletName().getStringValue(), 
					new ArrayList<String>());
			servletMapping.addUrlPattern(servletMappingType.getUrlPattern().getStringValue());
			dd.servletsMapping.add(servletMapping);
		}
	}


	private static void loadServlets(WebXmlSEA dd, WebAppDocument document) {
		List<ServletType> servlets = document.getWebApp().getServletList();
		ServletType servletType = null;
		Servlet servlet = null;
		Iterator<ServletType> it = servlets.iterator();
		while (it.hasNext()){
			servletType = it.next();
			servlet = new Servlet(
					servletType.getServletName().getStringValue(),
					servletType.getServletClass().getStringValue());
			addParamsToServlet(servletType, servlet);
			XsdIntegerType xsdLoadOnStartup = servletType.getLoadOnStartup();
			if (null!=xsdLoadOnStartup){
				servlet.setLoadOnStartup(new Integer(xsdLoadOnStartup.getStringValue()));
			}
			dd.servlets.add(servlet);
		}
		
	}


	private static void addParamsToServlet(ServletType servletType,
			Servlet servlet) {
		List<ParamValueType> paramList = servletType.getInitParamList();
		Iterator<ParamValueType> itParamValues = paramList.iterator();
		ParamValueType paramValue = null;
		InitParam initParam = null;
		while (itParamValues.hasNext()){
			paramValue = itParamValues.next();
			initParam = new InitParam(
					paramValue.getParamName().getStringValue(),
					paramValue.getParamValue().getStringValue());
			servlet.addInitParam(initParam);
		}
	}


	private static void loadFiltersMapping(WebXmlSEA dd, WebAppDocument document) {
		List<FilterMappingType> filtersMapping = document.getWebApp().getFilterMappingList();
		FilterMappingType filterMappingType = null;
		FilterMapping filterMapping = null;
		Iterator<FilterMappingType> it = filtersMapping.iterator();
		while (it.hasNext()){
			filterMappingType = it.next();
			filterMapping = new FilterMapping(
					filterMappingType.getFilterName().getStringValue(), 
					new ArrayList<String>());
			
			
			
			filterMapping.addUrlPattern(filterMappingType.getUrlPattern().getStringValue());
			
			dd.filtersMapping.add(filterMapping);
		}
	}


	@SuppressWarnings("deprecation")
	private static void loadFilters(WebXmlSEA dd, WebAppDocument document) {
		List<FilterType> filters = document.getWebApp().getFilterList();
		FilterType filterType = null;
		Filter filter = null;
		Iterator<FilterType> it = filters.iterator();
		while (it.hasNext()){
			filterType = it.next();
			filter = new Filter(
					(filterType.getDescriptionArray()[0]).getStringValue(), 
					(filterType.getDisplayNameArray()[0]).getStringValue(), 
					filterType.getFilterName().getStringValue(), 
					filterType.getFilterClass().getStringValue());
			dd.filters.add(filter);
		}
	}

	@SuppressWarnings("deprecation")
	private static void loadEnvEntries(WebXmlSEA dd, WebAppDocument document) {
		List<EnvEntryType> entries = document.getWebApp().getEnvEntryList();
		Iterator<EnvEntryType> it = entries.iterator();
		EnvEntryType entry = null;
		EnvEntry seaEntry = null;
		while (it.hasNext()){
			entry = it.next();
			seaEntry = new EnvEntry(
					(entry.getDescriptionArray()[0]).getStringValue(), 
					entry.getEnvEntryName().getStringValue(), 
					entry.getEnvEntryType().getStringValue(), 
					entry.getEnvEntryValue().getStringValue());
			dd.envEntries.add(seaEntry);
		}
	}

	public Collection<EnvEntry> getEnvEntries() {
		return envEntries;
	}

	public Collection<Filter> getFilters() {
		return filters;
	}


	public Collection<FilterMapping> getFiltersMapping() {
		return filtersMapping;
	}


	public Collection<Servlet> getServlets() {
		return servlets;
	}


	public Collection<ServletMapping> getServletsMapping() {
		return servletsMapping;
	}


	public Collection<Listener> getListeners() {
		return listeners;
	}

}
